@@ -50,6 +50,7 @@ OPTS = [
                help='Ceph username (ie: admin without "client." prefix).'),
     cfg.StrOpt('ceph_secret', help='Ceph key', secret=True),
     cfg.StrOpt('ceph_keyring', help='Ceph keyring path.'),
+    cfg.IntOpt('ceph_timeout', help='Ceph connection timeout'),
     cfg.StrOpt('ceph_conffile',
                default='/etc/ceph/ceph.conf',
                help='Ceph configuration file.'),
@@ -57,6 +58,9 @@ OPTS = [
 
 
 class CephStorage(_carbonara.CarbonaraBasedStorage):
+
+    WRITE_FULL = False
+
     def __init__(self, conf):
         super(CephStorage, self).__init__(conf)
         self.pool = conf.ceph_pool
@@ -65,6 +69,10 @@ class CephStorage(_carbonara.CarbonaraBasedStorage):
             options['keyring'] = conf.ceph_keyring
         if conf.ceph_secret:
             options['key'] = conf.ceph_secret
+        if conf.ceph_timeout:
+            options['rados_osd_op_timeout'] = conf.ceph_timeout
+            options['rados_mon_op_timeout'] = conf.ceph_timeout
+            options['client_mount_timeout'] = conf.ceph_timeout
 
         if not rados:
             raise ImportError("No module named 'rados' nor 'cradox'")
@@ -217,7 +225,7 @@ class CephStorage(_carbonara.CarbonaraBasedStorage):
         measures = []
         for n in object_names:
             data = self._get_object_content(n)
-            measures.extend(self._unserialize_measures(data))
+            measures.extend(self._unserialize_measures(n, data))
 
         yield measures
 
@@ -233,9 +241,11 @@ class CephStorage(_carbonara.CarbonaraBasedStorage):
             self.ioctx.aio_remove(n)
 
     @staticmethod
-    def _get_object_name(metric, timestamp_key, aggregation, granularity):
-        return str("gnocchi_%s_%s_%s_%s" % (
+    def _get_object_name(metric, timestamp_key, aggregation, granularity,
+                         version=3):
+        name = str("gnocchi_%s_%s_%s_%s" % (
             metric.id, timestamp_key, aggregation, granularity))
+        return name + '_v%s' % version if version else name
 
     def _object_exists(self, name):
         try:
@@ -251,17 +261,20 @@ class CephStorage(_carbonara.CarbonaraBasedStorage):
         else:
             self.ioctx.write_full(name, "metric created")
 
-    def _store_metric_measures(self, metric, timestamp_key,
-                               aggregation, granularity, data):
+    def _store_metric_measures(self, metric, timestamp_key, aggregation,
+                               granularity, data, offset=None, version=3):
         name = self._get_object_name(metric, timestamp_key,
-                                     aggregation, granularity)
-        self.ioctx.write_full(name, data)
+                                     aggregation, granularity, version)
+        if offset is None:
+            self.ioctx.write_full(name, data)
+        else:
+            self.ioctx.write(name, data, offset=offset)
         self.ioctx.set_xattr("gnocchi_%s_container" % metric.id, name, "")
 
     def _delete_metric_measures(self, metric, timestamp_key, aggregation,
-                                granularity):
+                                granularity, version=3):
         name = self._get_object_name(metric, timestamp_key,
-                                     aggregation, granularity)
+                                     aggregation, granularity, version)
         self.ioctx.rm_xattr("gnocchi_%s_container" % metric.id, name)
         self.ioctx.aio_remove(name)
 
@@ -276,10 +289,11 @@ class CephStorage(_carbonara.CarbonaraBasedStorage):
         for name in ('container', 'none'):
             self.ioctx.aio_remove("gnocchi_%s_%s" % (metric.id, name))
 
-    def _get_measures(self, metric, timestamp_key, aggregation, granularity):
+    def _get_measures(self, metric, timestamp_key, aggregation, granularity,
+                      version=3):
         try:
             name = self._get_object_name(metric, timestamp_key,
-                                         aggregation, granularity)
+                                         aggregation, granularity, version)
             return self._get_object_content(name)
         except rados.ObjectNotFound:
             if self._object_exists("gnocchi_%s_container" % metric.id):
@@ -287,27 +301,39 @@ class CephStorage(_carbonara.CarbonaraBasedStorage):
             else:
                 raise storage.MetricDoesNotExist(metric)
 
-    def _list_split_keys_for_metric(self, metric, aggregation, granularity):
+    def _list_split_keys_for_metric(self, metric, aggregation, granularity,
+                                    version=None):
         try:
             xattrs = self.ioctx.get_xattrs("gnocchi_%s_container" % metric.id)
         except rados.ObjectNotFound:
             raise storage.MetricDoesNotExist(metric)
-        keys = []
+        keys = set()
         for xattr, value in xattrs:
-            _, metric_id, key, agg, g = xattr.split('_', 4)
-            if aggregation == agg and granularity == float(g):
-                keys.append(key)
-
+            meta = xattr.split('_')
+            if (aggregation == meta[3] and granularity == float(meta[4]) and
+                    self._version_check(xattr, version)):
+                keys.add(meta[2])
         return keys
 
-    def _get_unaggregated_timeserie(self, metric):
+    @staticmethod
+    def _build_unaggregated_timeserie_path(metric, version):
+        return (('gnocchi_%s_none' % metric.id)
+                + ("_v%s" % version if version else ""))
+
+    def _get_unaggregated_timeserie(self, metric, version=3):
         try:
-            return self._get_object_content("gnocchi_%s_none" % metric.id)
+            return self._get_object_content(
+                self._build_unaggregated_timeserie_path(metric, version))
         except rados.ObjectNotFound:
             raise storage.MetricDoesNotExist(metric)
 
-    def _store_unaggregated_timeserie(self, metric, data):
-        self.ioctx.write_full("gnocchi_%s_none" % metric.id, data)
+    def _store_unaggregated_timeserie(self, metric, data, version=3):
+        self.ioctx.write_full(
+            self._build_unaggregated_timeserie_path(metric, version), data)
+
+    def _delete_unaggregated_timeserie(self, metric, version=3):
+        self.ioctx.aio_remove(
+            self._build_unaggregated_timeserie_path(metric, version))
 
     def _get_object_content(self, name):
         offset = 0
@@ -319,25 +345,3 @@ class CephStorage(_carbonara.CarbonaraBasedStorage):
             content += data
             offset += len(data)
         return content
-
-    # The following methods deal with Gnocchi <= 1.3 archives
-    def _get_metric_archive(self, metric, aggregation):
-        """Retrieve data in the place we used to store TimeSerieArchive."""
-        try:
-            return self._get_object_content(
-                str("gnocchi_%s_%s" % (metric.id, aggregation)))
-        except rados.ObjectNotFound:
-            raise storage.AggregationDoesNotExist(metric, aggregation)
-
-    def _store_metric_archive(self, metric, aggregation, data):
-        """Stores data in the place we used to store TimeSerieArchive."""
-        self.ioctx.write_full(
-            str("gnocchi_%s_%s" % (metric.id, aggregation)), data)
-
-    def _delete_metric_archives(self, metric):
-        for aggregation in metric.archive_policy.aggregation_methods:
-            try:
-                self.ioctx.remove_object(
-                    str("gnocchi_%s_%s" % (metric.id, aggregation)))
-            except rados.ObjectNotFound:
-                pass
